/*
    $Id: helper.c,v 1.4 2001/11/01 20:55:50 sensei Exp $

    Copyright (C) 2000-2001  Marko Zivanovic
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
     
#ifndef lint
static char vcid[] __attribute__ ((__unused__)) = "$Id: helper.c,v 1.4 2001/11/01 20:55:50 sensei Exp $";
#endif /* lint */

#include "helper.h"

/*  ************************************************************ */
/*  lookup host name                                             */
/*                                                               */
/*  parameters:                                                  */
/*     in   -   adress of host to lookup (IP or FQDN)            */
/*                                                               */
/*  return:                                                      */
/*  char*   -   FQDN or dot-quadded IP address                   */
/*  ************************************************************ */
char *hostlookup (unsigned long int in)
{ 
  static char res[1024];
  struct in_addr addr;
  struct hostent *host;
  
  addr.s_addr = in;
  host = NULL;

  if (nodns == 0)
    host = gethostbyaddr ((char *) &addr, sizeof (struct in_addr), AF_INET);

  if (host == NULL)
    strcpy (res, inet_ntoa (addr));
  else
    strcpy(res, host->h_name);

  return res;
}

/*  ************************************************************ */
/*  lookup service name, or, if not found, return port number    */
/*                                                               */
/*  parameters:                                                  */
/*     port   -   port number to lookup                          */
/*     proto  -   protocol (tcp, udp)                            */
/*                                                               */
/*  return                                                       */
/*     char*  -   service name or port number, if name not found */
/*  ************************************************************ */
char *serv_name (const int port, const char *proto)
{
  struct servent *se;
  char *serv = (char*) malloc (128);
  
  se = getservbyport (port, proto);
  if (se == NULL) {
    sprintf (serv, "%d", ntohs (port));
  } else
    strcpy (serv, se->s_name);
  
  return serv;
}

/*  ************************************************************ */
/*  lookup protocol name, or, if not found, return protocol      */
/*  number                                                       */
/*                                                               */
/*  parameters:                                                  */
/*     protocol   -   protocol number to lookup                  */
/*                                                               */
/*  return                                                       */
/*     char*  -   protocol name or number, if name not found     */
/*  ************************************************************ */
char *proto_name (int protocol)
{
  struct protoent *pe = NULL;
  static char name[128];

  pe = getprotobynumber (protocol);

  if (pe == NULL) {
    sprintf (name, "%d", protocol);
    return name;
  }

  return pe->p_name;
}

/*  ************************************************************ */
/*  lookup ethernet protocol name given protocol number          */
/*  NOTE: protocol information taken from linux/if_ether.h       */
/*                                                               */
/*  parameters:                                                  */
/*     protocol   -   ethernet protocol number                   */
/*                                                               */
/*  return:                                                      */
/*     char*      -   ethernet protocol name                     */
/*  ************************************************************ */
char *eth_proto (int protocol)
{
  static char res[128];

  switch (protocol) {
  case 0x0060: return "Ethernet Loopback packet";
  case 0x0200: return "Ethernet Echo packet";
  case 0x0400: return "Xerox PUP packet";
  case 0x0800: return "Internet Protocol packet";
  case 0x0805: return "CCITT X.25";
  case 0x0806: return "Address Resolution packet";
  case 0x08ff: return "G8BPQ AX.25 Ethernet Packet  [ NOT AN OFFICIALLY REGISTERED ID ]";
  case 0x6000: return "DEC Assigned proto";
  case 0x6001: return "DEC DNA Dump/Load";
  case 0x6002: return "DEC DNA Remote Console";
  case 0x6003: return "DEC DNA Routing";
  case 0x6004: return "DEC LAT";
  case 0x6005: return "DEC Diagnostics";
  case 0x6006: return "DEC Customer use";
  case 0x6007: return "DEC Systems Comms Arch";
  case 0x8035: return "Reverse Addr Res packet";
  case 0x809b: return "Appletalk DDP";
  case 0x80f3: return "Appletalk AARP";
  case 0x8137: return "IPX over DIX";
  case 0x86dd: return "IPv6 over bluebook";
  } /* switch */

  sprintf (res, "%d", protocol);
  return res;
}

/*  ************************************************************ */
/*  return icmp packet type descriptive text                     */
/*  NOTE: icmp information taken from linux/icmp.h               */
/*                                                               */
/*  parameters:                                                  */
/*     type    -    icmp type                                    */
/*                                                               */
/*  return:                                                      */ 
/*     char*   -    icmp type descriptive text                   */
/*  ************************************************************ */
char *icmp_type (int type)
{
  static char res[128];

  switch (type) {
  case 0: return "echo reply";
  case 3: return "destination unreachable";
  case 4: return "source quench";
  case 5: return "redirect";
  case 8: return "echo request";
  case 11: return "time exceeded";
  case 12: return "parameter problem";
  case 13: return "timestamp request";
  case 14: return "timestamp reply";
  case 15: return "information request";
  case 16: return "information reply";
  case 17: return "address mask request";
  case 18: return "address mask reply";
  }

  sprintf (res, "%d", type);
  return res;
}

/*  ************************************************************ */
/*  return descriptive text for every icmp packet type sub-code  */
/*  NOTE: icmp information taken from linux/icmp.h               */
/*                                                               */
/*  parameters:                                                  */
/*      type    -    icmp packet type                            */
/*      code    -    icmp packet type sub-code                   */
/*                                                               */
/*  return:                                                      */
/*      char*   -    icmp packet type sub-code descriptive text  */
/*  ************************************************************ */
char *icmp_code (int type, int code)
{
  static char res[128];

  /* unreachable codes */
  if (type == 3) {
    switch (code) {
    case 0: return "network unreachable";
    case 1: return "host unreachable";
    case 2: return "protocol unreachable";
    case 3: return "port unreachable";
    case 4: return "fragmentation needed";
    case 5: return "source route failed";
    case 6: return "network unknown";
    case 7: return "host unknown";
    case 8: return "host isolated";
    case 9: return "network ?ANO?";
    case 10: return "host ?ANO?";
    case 11: return "network unreachable for type-of-service";
    case 12: return "host unreachable for type-of-service";
    case 13: return "packet filtered";
    case 14: return "precedence violation";
    case 15: return "precedence cut off";
    } /* switch destination unreachable code */
  } /* if type == ICMP_DEST_UNREACH */
  else if (type == 5) {
    switch (code) {
    case 0: return "redirect network";
    case 1: return "redirect host";
    case 2: return "redirect network for type-of-service";
    case 3: return "redirect host for type-of-service";
    } /* switch redirect code */
  } /* if type == ICMP_REDIRECT */
  else if (type == 11) {
    switch (code) {
    case 0: return "time-to-live count exceeded";
    case 1: return "fragment reassembly time exceeded";
    } /* switch time exceeded code */
  } /* if type == ICMP_TIME_EXCEEDED */
  
  sprintf (res, "%d", code);
  return res;
}

char *eth_ntoa (unsigned char addr[ETH_ALEN])
{
  static char res[20];
  char tmp[3];
  int i;

  res[0] = 0;

  for (i = 0; i < ETH_ALEN - 1; i++) {
    sprintf (tmp, "%.2X:", addr[i]);
    strcat (res, tmp);
  }
  sprintf (tmp, "%.2X", addr[ETH_ALEN - 1]);
  strcat (res, tmp);

  return res;
}

/*  ************************************************************ */
/*  show help text                                               */
/*                                                               */
/*  no parameters and no return                                  */
/*  ************************************************************ */
void show_help (void)
{
  printf ("\n%s recognizes following command line options:\n", PACKAGE);
  printf ("  --help, -h\n");
  printf ("     displays this text.\n\n");
  printf ("  --version, -v\n");
  printf ("     displays version information and copyright notice.\n\n");
  printf ("  --debug <c>, -d <c>\n");
  printf ("     Turns on specific debug mode (<c> can be one of the following):\n");
  printf ("       c   -   configuration file\n");
  printf ("       m   -   memory allocation\n");
  printf ("       b   -   don't fork sniffer into background\n\n");
  printf ("  --quiet, -q\n");
  printf ("     No console output will be generated\n\n");
  printf ("  --host-name-lookup, -n\n");
  printf ("     Turns on host name lookup for generated logs.\n");
  printf ("     WARNING: turning this option on without proper filters will\n");
  printf ("              initiate chain reaction: every captured IP packet will\n");
  printf ("              trigger host lookup which generates more IP packets that\n");
  printf ("              require host lookup and so on. Your log file will keep\n");
  printf ("              growing and sniffer will enter deadlock!\n");
  printf ("\n");
  exit (0);
}

/*  ************************************************************ */
/*  show version text                                            */
/*                                                               */
/*  no parameters and no return                                  */
/*  ************************************************************ */
void show_version (void)
{
  printf ("\n%s %s\n", PACKAGE, VERSION);

  printf ("Copyright (C) 2000-2001 Marko Zivanovic.\n");
  printf ("%s comes with NO WARRANTY,\n", PACKAGE);
  printf ("to the extent permitted by law.\n");
  printf ("You may redistribute copies of %s\n", PACKAGE);
  printf ("under the terms of the GNU General Public License.\n");
  printf ("For more information about these matters,\n");
  printf ("see the files named COPYING.\n\n");
  printf ("Report bugs to sensei@BankerInter.net\n\n");

  exit (0);
}
