/*
    $Id: sighandlers.c,v 1.3 2001/11/01 20:55:50 sensei Exp $

    Copyright (C) 2000-2001  Marko Zivanovic
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef lint
static char vcid[] __attribute__ ((__unused__)) = "$Id: sighandlers.c,v 1.3 2001/11/01 20:55:50 sensei Exp $";
#endif /* lint */

#include "sighandlers.h"

/*  ************************************************************ */
/*  signal working threads that they should shutdown             */
/*                                                               */
/*  parameters:                                                  */
/*     sig   -   signal number received by  process              */
/*  ************************************************************ */
RETSIGTYPE shutdown_sniffer (int sig)
{
  static int c;
  if (c == 3) c = 0;
  c++;
  if (c != 1) return;

  /* instruct sniffer thread to close socket and shutdown */
  stop_sniffer = 1;

  /* instruct dumper thread to dump packets remaining in buffer and
     shutdown */
  stop_dumper = 1;
}

/*  ************************************************************ */
/*  enable memory allocation debugging: all memory allocation    */
/*  related actions will be logged to file specified by          */
/*  MALLOC_TRACE environment variable                            */
/*  NOTE: This is GNU extension and on non-GNU systems it will   */
/*  silently do nothing                                          */
/*                                                               */
/*  parameters:                                                  */
/*     sig   -   signal that was sent to process                 */
/*  ************************************************************ */
RETSIGTYPE enable_malloc_debug (int sig)
{
#ifdef HAVE_MTRACE
  mtrace ();
#endif
  logmsg (LOG_INFO, "debug mode enabled");
  signal (SIGUSR1, enable_malloc_debug);
}

/*  ************************************************************ */
/*  disable previously enabled memory allocation debugging;      */
/*  if malloc debugging is not enabled it will do nothing        */
/*                                                               */
/*  parameters:                                                  */
/*     sig   -   signal that was sent to process                 */
/*  ************************************************************ */
RETSIGTYPE disable_malloc_debug (int sig)
{
#ifdef HAVE_MTRACE
  muntrace ();
#endif
  logmsg (LOG_INFO, "debug mode disabled");
  signal (SIGUSR2, disable_malloc_debug);
}

/*  ************************************************************ */
/*  reload configuration from file and reinitialize all engines  */
/*                      NOT YET IMPLEMENTED                      */
/*                                                               */
/*  parameters:                                                  */
/*     sig   -   signal that was sent to process                 */
/*  ************************************************************ */
RETSIGTYPE reload_configuration (int sig)
{
}

/*  ************************************************************ */
/*  dump current status info to log file and/or console          */
/*                                                               */
/*  parameters:                                                  */
/*     sig   -   signal sent to process                          */
/*                                                               */
/*  no return                                                    */
/*  ************************************************************ */
RETSIGTYPE print_status (int sig)
{
  static int c;
  time_t now;
  long up;
  long day, hour, min, sec;
  
  if (c == 3) c = 0;
  c++;
  if (c != 1) return;
  
  time (&now);
  up = difftime (now, start_time);
  sec = up;
  day = sec / 86400;
  sec -= day * 86400;
  hour = sec / 3600;
  sec -= hour * 3600;
  min = sec / 60;
  sec -= min * 60;

  logmsg (LOG_INFO, "uptime: %ld days, %ld hours, %ld minutes and %ld seconds", day, hour, min, sec);
  logmsg (LOG_INFO, "buffer size: %ld KB", buffer_length / 1024);
  logmsg (LOG_INFO, "packets currently in buffer: %ld", no_of_packets_in_buffer);
  logmsg (LOG_INFO, "packets buffered: %ld", no_of_packets);
  logmsg (LOG_INFO, "packets processed: %ld", total_packets);
  logmsg (LOG_INFO, "average load: %ld packets/s, %ld KB/s", no_of_packets / up, total_buff_len / up / 1024);
}

/*  ************************************************************ */
/*  instruct sniffer to dump current content of the buffer as    */
/*  soon as possible                                             */
/*                                                               */
/*  parameters:                                                  */
/*     sig   -   signal sent to process                          */
/*                                                               */
/*  no return                                                    */
/*  ************************************************************ */
RETSIGTYPE dump_buffer_now (int sig)
{
  static int c;

  if (c == 3) c = 0;
  c++;
  if (c != 1) return;
  
  immediate_dump = 1;
}
